//Project: WinchBot 2.0
//Homepage: www.HomoFaciens.de
//Author Norbert Heinz
//Version: 0.1
//Creation date: 03.06.2017
//
//compile with gcc WinchBot2.c -o WinchBot2 -lmraa -I/usr/local/include -L/usr/local/lib -lm -pthread -Wall
//For details see:
//http://www.HomoFaciens.de/technics-machines-winchbot2_en_navion.htm

#include "mraa.h"
#include <stdio.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include <dirent.h>
#include <math.h>
#include <unistd.h>

#include <signal.h>



//Base length of triangle in mm
#define BASELENGTH          895.0
//Initial cord length in mm
#define INITIAL_LENGTH       600.0
//Distance mount point cords to center of ball bearing mount at triangle
#define INITIAL_BAR_LENGTH   373.0
//Mount point cords on square tube
#define RADIUS_CORD_MOUNT     23.0
//Height from pivot point servo 4 to gripper
#define LEVER_S5 50.0
//Horizontal distance from pivot point servo 4 to tip of gripper
#define GRIPPER_LENGTH 60.0
//Horizontal offset from pivot point Servo 3 to centerline gripper
#define GRIPPER_OFFSET  5.0

//Time between two steps of platform motors
#define P_PAUSE 2500L

//Time between two steps of WinchBot
#define STEP_PAUSE 100L
//Pause needed for high signal on step input of motor driver
#define STEP_COMMAND_PAUSE 50L

#define PI 3.1415927

//Steps per 1 degree of platform: 200 steps (400 half steps) per turn, 7 teeth to 70 teeth
#define STEPS_PER_ANGLE  11.11111112

#define M1_STEP          2            //PWM output motor
#define M1_DIR           4            //Direction output motor
#define M2_STEP          7            //PWM output motor
#define M2_DIR           8            //Direction output motor
#define M3_STEP          9            //PWM output motor
#define M3_DIR          10            //Direction output motor


#define ENABLE_MOTORS    19            //Motors disabled if pin LOW


#define MOTOR_P1_A   11
#define MOTOR_P1_B   12
#define MOTOR_P1_C   13
#define MOTOR_P1_D   14

#define MOTOR_P2_A   15
#define MOTOR_P2_B   16
#define MOTOR_P2_C   17
#define MOTOR_P2_D   18


#define SERVO_3  3            //Servo arm rotate
#define SERVO_4  5            //Servo arm tilt
#define SERVO_5  6            //Servo gripper


#define SERVO_3_MIN  0
#define SERVO_3_MAX  180

#define SERVO_4_MIN  0
#define SERVO_4_MAX  180

#define SERVO_5_MIN  0
#define SERVO_5_MAX  180


int run = 1;
//24 teeth (motor gear) to 80 teeth (drum gear) 2000 steps per motor revolution, diameter drum 51.5mm
double StepsPermm = 6666.6 / 162.0;

double length1, length2, length3;


int Servo3 = 90;
int Servo4 = 90;
int Servo5 = 90;

#define SERVO_3_OFFSET  0
#define SERVO_4_OFFSET  30
#define SERVO_5_OFFSET  0

long currentX = 0, currentY = 0, currentZ = 0;
long StepPause = STEP_PAUSE;

double M1X, M1Y, M1Z;
double M2X, M2Y, M2Z;
double M3X, M3Y, M3Z;

int MotorP1Step = 0;
int MotorP2Step = 0;
double AngleP1 = 0.0;
double AngleP2 = 0.0;


mraa_result_t gpioRet = MRAA_SUCCESS;
mraa_gpio_context gpioM1dir;
mraa_gpio_context gpioM1step;
mraa_gpio_context gpioM2dir;
mraa_gpio_context gpioM2step;
mraa_gpio_context gpioM3dir;
mraa_gpio_context gpioM3step;
mraa_gpio_context gpioP1a;
mraa_gpio_context gpioP1b;
mraa_gpio_context gpioP1c;
mraa_gpio_context gpioP1d;
mraa_gpio_context gpioP2a;
mraa_gpio_context gpioP2b;
mraa_gpio_context gpioP2c;
mraa_gpio_context gpioP2d;

mraa_gpio_context gpioEnable;

mraa_pwm_context pwmServo3;
mraa_pwm_context pwmServo4;
mraa_pwm_context pwmServo5;


//Catch term signal
void stop(int signum){
   run = 0;
}


//++++++++++++++++++++++++++++++ move servo 3 (arm rotate) ++++++++++++++++++++++++++++++++++++++++++++++++
void moveServo3(int direction){

  if((Servo3 < SERVO_3_MAX && direction > 0) || (Servo3 > SERVO_3_MIN && direction < 0)){
    Servo3 += direction;
    mraa_pwm_pulsewidth_us (pwmServo3, (Servo3 + SERVO_3_OFFSET) * 1000 / 90 + 500);
    //gpioServo(SERVO_3, (Servo3 + SERVO_3_OFFSET) * 1000 / 90 + 500);
  }
}

//++++++++++++++++++++++++++++++ move servo 4 (arm tilt) ++++++++++++++++++++++++++++++++++++++++++++++++++
void moveServo4(int direction){

  if((Servo4 < SERVO_4_MAX && direction > 0) || (Servo4 > SERVO_4_MIN && direction < 0)){
    Servo4 += direction;
    mraa_pwm_pulsewidth_us (pwmServo4, (Servo4 + SERVO_4_OFFSET) * 1000 / 90 + 500);
    //gpioServo(SERVO_4, (Servo4 + SERVO_4_OFFSET) * 1000 / 90 + 500);
  }
}

//++++++++++++++++++++++++++++++ move servo 5 (gripper) +++++++++++++++++++++++++++++++++++++++++++++++++++
void moveServo5(int direction){

  if((Servo5 < SERVO_5_MAX && direction > 0) || (Servo5 > SERVO_5_MIN && direction < 0)){
    Servo5 += direction;
    mraa_pwm_pulsewidth_us (pwmServo5, (Servo5 + SERVO_5_OFFSET) * 1000 / 90 + 500);
    //gpioServo(SERVO_5, (Servo5 + SERVO_5_OFFSET) * 1000 / 90 + 500);
  }
}

//++++++++++++++++++++++++++++++ move motor P1 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void motorP1(long steps, long uPause){
  long i;

  AngleP1 += steps;

  //printf("Moving motor1 for %ld steps\n", steps);


  for(i=0; i<abs(steps); i++){
    if(steps > 0){
      MotorP1Step++;
    }
    else{
      MotorP1Step--;
    }
    if(MotorP1Step > 7){
      MotorP1Step = 0;
    }
    if(MotorP1Step < 0){
      MotorP1Step = 7;
    }

    if(MotorP1Step == 0){
      gpioRet = mraa_gpio_write(gpioP1a, 1);
      gpioRet = mraa_gpio_write(gpioP1b, 0);
      gpioRet = mraa_gpio_write(gpioP1c, 0);
      gpioRet = mraa_gpio_write(gpioP1d, 0);
    }

    if(MotorP1Step == 1){
      gpioRet = mraa_gpio_write(gpioP1a, 1);
      gpioRet = mraa_gpio_write(gpioP1b, 0);
      gpioRet = mraa_gpio_write(gpioP1c, 1);
      gpioRet = mraa_gpio_write(gpioP1d, 0);
    }

    if(MotorP1Step == 2){
      gpioRet = mraa_gpio_write(gpioP1a, 0);
      gpioRet = mraa_gpio_write(gpioP1b, 0);
      gpioRet = mraa_gpio_write(gpioP1c, 1);
      gpioRet = mraa_gpio_write(gpioP1d, 0);
    }

    if(MotorP1Step == 3){
      gpioRet = mraa_gpio_write(gpioP1a, 0);
      gpioRet = mraa_gpio_write(gpioP1b, 1);
      gpioRet = mraa_gpio_write(gpioP1c, 1);
      gpioRet = mraa_gpio_write(gpioP1d, 0);
    }

    if(MotorP1Step == 4){
      gpioRet = mraa_gpio_write(gpioP1a, 0);
      gpioRet = mraa_gpio_write(gpioP1b, 1);
      gpioRet = mraa_gpio_write(gpioP1c, 0);
      gpioRet = mraa_gpio_write(gpioP1d, 0);
    }

    if(MotorP1Step == 5){
      gpioRet = mraa_gpio_write(gpioP1a, 0);
      gpioRet = mraa_gpio_write(gpioP1b, 1);
      gpioRet = mraa_gpio_write(gpioP1c, 0);
      gpioRet = mraa_gpio_write(gpioP1d, 1);
    }

    if(MotorP1Step == 6){
      gpioRet = mraa_gpio_write(gpioP1a, 0);
      gpioRet = mraa_gpio_write(gpioP1b, 0);
      gpioRet = mraa_gpio_write(gpioP1c, 0);
      gpioRet = mraa_gpio_write(gpioP1d, 1);
    }

    if(MotorP1Step == 7){
      gpioRet = mraa_gpio_write(gpioP1a, 1);
      gpioRet = mraa_gpio_write(gpioP1b, 0);
      gpioRet = mraa_gpio_write(gpioP1c, 0);
      gpioRet = mraa_gpio_write(gpioP1d, 1);
    }
    usleep(uPause);
  }

}

//++++++++++++++++++++++++++++++ move motor P2 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void motorP2(long steps, long uPause){
  long i;

  AngleP2 += steps;

  //printf("Moving motor1 for %ld steps\n", steps);


  for(i=0; i<abs(steps); i++){
    if(steps > 0){
      MotorP2Step++;
    }
    else{
      MotorP2Step--;
    }
    if(MotorP2Step > 7){
      MotorP2Step = 0;
    }
    if(MotorP2Step < 0){
      MotorP2Step = 7;
    }

    if(MotorP2Step == 0){
      gpioRet = mraa_gpio_write(gpioP2a, 1);
      gpioRet = mraa_gpio_write(gpioP2b, 0);
      gpioRet = mraa_gpio_write(gpioP2c, 0);
      gpioRet = mraa_gpio_write(gpioP2d, 0);
    }

    if(MotorP2Step == 1){
      gpioRet = mraa_gpio_write(gpioP2a, 1);
      gpioRet = mraa_gpio_write(gpioP2b, 0);
      gpioRet = mraa_gpio_write(gpioP2c, 1);
      gpioRet = mraa_gpio_write(gpioP2d, 0);
    }

    if(MotorP2Step == 2){
      gpioRet = mraa_gpio_write(gpioP2a, 0);
      gpioRet = mraa_gpio_write(gpioP2b, 0);
      gpioRet = mraa_gpio_write(gpioP2c, 1);
      gpioRet = mraa_gpio_write(gpioP2d, 0);
    }

    if(MotorP2Step == 3){
      gpioRet = mraa_gpio_write(gpioP2a, 0);
      gpioRet = mraa_gpio_write(gpioP2b, 1);
      gpioRet = mraa_gpio_write(gpioP2c, 1);
      gpioRet = mraa_gpio_write(gpioP2d, 0);
    }

    if(MotorP2Step == 4){
      gpioRet = mraa_gpio_write(gpioP2a, 0);
      gpioRet = mraa_gpio_write(gpioP2b, 1);
      gpioRet = mraa_gpio_write(gpioP2c, 0);
      gpioRet = mraa_gpio_write(gpioP2d, 0);
    }

    if(MotorP2Step == 5){
      gpioRet = mraa_gpio_write(gpioP2a, 0);
      gpioRet = mraa_gpio_write(gpioP2b, 1);
      gpioRet = mraa_gpio_write(gpioP2c, 0);
      gpioRet = mraa_gpio_write(gpioP2d, 1);
    }

    if(MotorP2Step == 6){
      gpioRet = mraa_gpio_write(gpioP2a, 0);
      gpioRet = mraa_gpio_write(gpioP2b, 0);
      gpioRet = mraa_gpio_write(gpioP2c, 0);
      gpioRet = mraa_gpio_write(gpioP2d, 1);
    }

    if(MotorP2Step == 7){
      gpioRet = mraa_gpio_write(gpioP2a, 1);
      gpioRet = mraa_gpio_write(gpioP2b, 0);
      gpioRet = mraa_gpio_write(gpioP2c, 0);
      gpioRet = mraa_gpio_write(gpioP2d, 1);
    }
    usleep(uPause);
  }

}

//++++++++++++++++++++++++++++++ move motor 1 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void motor1(long steps, long uPause){
  long i;
  length1 += steps;

  //printf("Moving motor1 for %ld steps\n", steps);

  if(steps < 0){
    gpioRet = mraa_gpio_write(gpioM1dir, 1);
  }
  else{
    gpioRet = mraa_gpio_write(gpioM1dir, 0);
  }

  if(steps != 0){
    for(i=0; i < labs(steps); i++){
      gpioRet = mraa_gpio_write(gpioM1step, 1);
      usleep(STEP_COMMAND_PAUSE);
      gpioRet = mraa_gpio_write(gpioM1step, 0);
      usleep(uPause);
    }
  }
}


//++++++++++++++++++++++++++++++ move motor 2 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void motor2(long steps, long uPause){
  long i;
  length2 += steps;

  //printf("Moving motor2 for %ld steps\n", steps);

  if(steps < 0){
    gpioRet = mraa_gpio_write(gpioM2dir, 1);
  }
  else{
    gpioRet = mraa_gpio_write(gpioM2dir, 0);
  }

  if(steps != 0){
    for(i=0; i < labs(steps); i++){
      gpioRet = mraa_gpio_write(gpioM2step, 1);
      usleep(STEP_COMMAND_PAUSE);
      gpioRet = mraa_gpio_write(gpioM2step, 0);
      usleep(uPause);
    }
  }
}

//++++++++++++++++++++++++++++++ move motor 3 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void motor3(long steps, long uPause){
  long i;
  length3 += steps;

  //printf("Moving motor3 for %ld steps\n", steps);

  if(steps < 0){
    gpioRet = mraa_gpio_write(gpioM3dir, 1);
  }
  else{
    gpioRet = mraa_gpio_write(gpioM3dir, 0);
  }

  if(steps != 0){
    for(i=0; i < labs(steps); i++){
      gpioRet = mraa_gpio_write(gpioM3step, 1);
      usleep(STEP_COMMAND_PAUSE);
      gpioRet = mraa_gpio_write(gpioM3step, 0);
      usleep(uPause);
    }
  }
}


//++++++++++++++++++++++++++++++++++++++ MoveToCoordinates +++++++++++++++++++++++++++++++++++++++++++++++++++++
int MoveToCoordinates(long X, long Y, long Z, int Servo3New, int Servo4New, int Servo5New, long sleepTime){
  double length1New, length2New, length3New;
  double M1MountX, M1MountY, M1MountZ;
  double M2MountX, M2MountY, M2MountZ;
  double M3MountX, M3MountY, M3MountZ;
//  long offsetX, offsetY, offsetZ;
//  long tempX, tempY, tempZ;
  long stepDiff = 0;
//  double alpha = 90.0, beta = 90.0;
//  double alpha3;
//  double alpha4;
//  long barMountZ;


  double alpha, beta;
  double AngleP1New, AngleP2New;
  double nowM1, nowM2, nowM3, nowP1, nowP2, nowS3, nowS4, nowS5;
  double diffM1, diffM2, diffM3, diffP1, diffP2, diffS3, diffS4, diffS5;


  printf("Moving to X=%ld, Y=%ld, Z=%ld, Servo3=%d, Servo4=%d, Servo5=%d\n", X, Y, Z, Servo3New, Servo4New, Servo5New);


  if(Servo3New > SERVO_3_MAX){
    Servo3New = SERVO_3_MAX;
  }
  if(Servo3New < SERVO_3_MIN){
    Servo3New = SERVO_3_MIN;
  }

  if(Servo4New > SERVO_4_MAX){
    Servo4New = SERVO_4_MAX;
  }
  if(Servo4New < SERVO_4_MIN){
    Servo4New = SERVO_4_MIN;
  }

  if(Servo5New > SERVO_5_MAX){
    Servo5New = SERVO_5_MAX;
  }
  if(Servo5New < SERVO_5_MIN){
    Servo5New = SERVO_5_MIN;
  }


  alpha = atan(Y/(INITIAL_BAR_LENGTH * StepsPermm - Z))*180 / PI;
  beta = atan(X/(INITIAL_BAR_LENGTH * StepsPermm - Z))*180 / PI;

  AngleP1New = round(alpha * STEPS_PER_ANGLE);
  AngleP2New = round(beta * STEPS_PER_ANGLE);



  M1MountX = X + RADIUS_CORD_MOUNT * StepsPermm * cos(30.0 * PI / 180.0) * cos(alpha) * cos(beta);
  M1MountY = Y - RADIUS_CORD_MOUNT * StepsPermm * sin(30.0 * PI / 180.0) * cos(alpha) * cos(beta);
  M1MountZ = Z - RADIUS_CORD_MOUNT * StepsPermm * sin(30.0 * PI / 180.0) * sin(alpha) * sin(beta);;

  M2MountX = X;
  M2MountY = Y + cos(alpha) * RADIUS_CORD_MOUNT * StepsPermm;
  M2MountZ = Z + sin(alpha) * RADIUS_CORD_MOUNT * StepsPermm;

  M3MountX = X - RADIUS_CORD_MOUNT * StepsPermm * cos(30.0 * PI / 180.0) * cos(alpha) * cos(beta);
  M3MountY = Y - RADIUS_CORD_MOUNT * StepsPermm * sin(30.0 * PI / 180.0) * cos(alpha) * cos(beta);
  M3MountZ = Z - RADIUS_CORD_MOUNT * StepsPermm * sin(30.0 * PI / 180.0) * sin(alpha) * sin(beta);;


  length1New = labs(sqrt(pow(M1X - M1MountX, 2.0) + pow(M1Y - M1MountY, 2.0) + pow(M1Z - M1MountZ, 2.0)));
  length2New = labs(sqrt(pow(M2X - M2MountX, 2.0) + pow(M2Y - M2MountY, 2.0) + pow(M2Z - M2MountZ, 2.0)));
  length3New = labs(sqrt(pow(M3X - M3MountX, 2.0) + pow(M3Y - M3MountY, 2.0) + pow(M3Z - M3MountZ, 2.0)));

  length1New = labs(sqrt(pow(M1X - X, 2.0) + pow(M1Y - Y, 2.0) + pow(M1Z - Z, 2.0)));
  length2New = labs(sqrt(pow(M2X - X, 2.0) + pow(M2Y - Y, 2.0) + pow(M2Z - Z, 2.0)));
  length3New = labs(sqrt(pow(M3X - X, 2.0) + pow(M3Y - Y, 2.0) + pow(M3Z - Z, 2.0)));

  stepDiff = abs(length1New - length1);
  if(abs(length2New - length2) > stepDiff){
    stepDiff = abs(length2New - length2);
  }
  if(abs(length3New - length3) > stepDiff){
    stepDiff = abs(length3New - length3);
  }
  if(abs(AngleP1New - AngleP1) > stepDiff){
    stepDiff = abs(AngleP1New - AngleP1);
  }
  if(abs(AngleP2New - AngleP2) > stepDiff){
    stepDiff = abs(AngleP2New - AngleP2);
  }
  if(abs(Servo3New - Servo3) > stepDiff){
    stepDiff = abs(Servo3New - Servo3);
  }
  if(abs(Servo4New - Servo4) > stepDiff){
    stepDiff = abs(Servo4New - Servo4);
  }
  if(abs(Servo5New - Servo5) > stepDiff){
    stepDiff = abs(Servo5New - Servo5);
  }

  printf("length1New - length1 = %lf, length2New - length2 = %lf, length3New - length3 = %lf, stepDiff=%ld, stepDiff*STEP_PAUSE=%ld\n", length1New - length1, length2New - length2, length3New - length3, stepDiff, stepDiff*STEP_PAUSE);




  if(abs(length1New - length1)!=0){
    diffM1 = (double)(stepDiff) / (double)(abs(length1New - length1));
  }
  else{
    diffM1 = stepDiff+1000;
  }

  if(abs(length2New - length2)!=0){
    diffM2 = (double)(stepDiff) / (double)(abs(length2New - length2));
  }
  else{
    diffM2 = stepDiff+1000;
  }

  if(abs(length3New - length3)!=0){
    diffM3 = (double)(stepDiff) / (double)(abs(length3New - length3));
  }
  else{
    diffM3 = stepDiff+1000;
  }


  if(abs(AngleP1New - AngleP1)!=0){
    diffP1 = (double)(stepDiff) / (double)(abs(AngleP1New - AngleP1));
  }
  else{
    diffP1 = stepDiff+1000;
  }

  if(abs(AngleP2New - AngleP2)!=0){
    diffP2 = (double)(stepDiff) / (double)(abs(AngleP2New - AngleP2));
  }
  else{
    diffP2 = stepDiff+1000;
  }


  if(abs(Servo3New - Servo3)!=0){
    diffS3 = (double)(stepDiff) / (double)(abs(Servo3New - Servo3));
  }
  else{
    diffS3 = stepDiff+1000;
  }

  if(abs(Servo4New - Servo4)!=0){
    diffS4 = (double)(stepDiff) / (double)(abs(Servo4New - Servo4));
  }
  else{
    diffS4 = stepDiff+1000;
  }

  if(abs(Servo5New - Servo5)!=0){
    diffS5 = (double)(stepDiff) / (double)(abs(Servo5New - Servo5));
  }
  else{
    diffS5 = stepDiff+1000;
  }


  nowM1 = diffM1;
  nowM2 = diffM2;
  nowM3 = diffM3;

  nowP1 = diffP1;
  nowP2 = diffP2;

  nowS3 = diffS3;
  nowS4 = diffS4;
  nowS5 = diffS5;

  //printf("diffM1=%5.2lf, diffM2=%5.2lf, diffM3=%5.2lf, diffP1=%5.2lf, diffP2=%5.2lf, diffS3=%5.2lf, diffS4=%5.2lf, diffS5=%5.2lf\n", diffM1, diffM2, diffM3, diffP1, diffP2, diffS3, diffS4, diffS5);


  for(double i=1.0; i<=stepDiff; i++){


  //printf("uTimeNow=%ld, uTimeStart=%ld, uTimeDiff=%ld (%ld)\n", uTimeNow, uTimeStart, uTimeDiff, stepDiff*STEP_PAUSE);

    if(nowM1 <= i){
      if(length1New > length1){
        motor1(1, 0);
      }
      if(length1New < length1){
        motor1(-1, 0);
      }
      nowM1+=diffM1;
    }

    if(nowM2 <= i){
      if(length2New > length2){
        motor2(1, 0);
      }
      if(length2New < length2){
        motor2(-1, 0);
      }
      nowM2+=diffM2;
    }

    if(nowM3 <= i){
      if(length3New > length3){
        motor3(1, 0);
      }
      if(length3New < length3){
        motor3(-1, 0);
      }
      nowM3+=diffM3;
    }

    if(nowP1 <= i){
      if(AngleP1New > AngleP1){
        motorP1(1, 0);
      }
      if(AngleP1New < AngleP1){
        motorP1(-1, 0);
      }
      nowP1+=diffP1;
    }

    if(nowP2 <= i){
      if(AngleP2New > AngleP2){
        motorP2(1, 0);
      }
      if(AngleP2New < AngleP2){
        motorP2(-1, 0);
      }
      nowP2+=diffP2;
    }

    if(nowS3 <= i){
      if(Servo3New > Servo3){
        moveServo3(1);
      }
      if(Servo3New < Servo3){
        moveServo3(-1);
      }
      nowS3+=diffS3;
    }

    if(nowS4 <= i){
      if(Servo4New > Servo4){
        moveServo4(1);
      }
      if(Servo4New < Servo4){
        moveServo4(-1);
      }
      nowS4+=diffS4;
    }

    if(nowS5 <= i){
      if(Servo5New > Servo5){
        moveServo5(1);
      }
      if(Servo5New < Servo5){
        moveServo5(-1);
      }
      nowS5+=diffS5;
    }

    usleep(StepPause);
  }


  int endPosOK=0;


  while (endPosOK==0){
    //printf("Remaining steps\n");
    endPosOK=1;
    if(length1New > length1){
      motor1(1, 0);
      endPosOK=0;
  //    printf("length1New=%lf, length1=%lf\n", length1New, length1);
    }
    if(length1New < length1){
      motor1(-1, 0);
      endPosOK=0;
  //    printf("length1New=%lf, length1=%lf\n", length1New, length1);
    }

    if(length2New > length2){
      motor2(1, 0);
      endPosOK=0;
  //    printf("length2New=%lf, length2=%lf\n", length2New, length2);
    }
    if(length2New < length2){
      motor2(-1, 0);
      endPosOK=0;
  //    printf("length2New=%lf, length2=%lf\n", length2New, length2);
    }

    if(length3New > length3){
      motor3(1, 0);
      endPosOK=0;
  //    printf("length3New=%lf, length3=%lf\n", length3New, length3);
    }
    if(length3New < length3){
      motor3(-1, 0);
      endPosOK=0;
  //    printf("length3New=%lf, length3=%lf\n", length3New, length3);
    }


    if(AngleP1New>AngleP1){
      motorP1(1, 0);
      endPosOK=0;
      //printf("AngleP1New=%lf, AngleP1=%lf\n", AngleP1New, AngleP1);
    }
    if(AngleP1New<AngleP1){
      motorP1(-1, 0);
      endPosOK=0;
      //printf("AngleP1New=%lf, AngleP1=%lf\n", AngleP1New, AngleP1);
    }

    if(AngleP2New>AngleP2){
      motorP2(1, 0);
      endPosOK=0;
      //printf("AngleP2New=%lf, AngleP2=%lf\n", AngleP2New, AngleP2);
    }
    if(AngleP2New<AngleP2){
      motorP2(-1, 0);
      endPosOK=0;
      //printf("AngleP2New=%lf, AngleP2=%lf\n", AngleP2New, AngleP2);
    }



    if(Servo3New>Servo3){
      moveServo3(1);
      endPosOK=0;
      //printf("ServoNew3=%d, Servo3=%d\n", Servo3New, Servo3);
    }
    if(Servo3New<Servo3){
      moveServo3(-1);
      endPosOK=0;
      //printf("ServoNew3=%d, Servo3=%d\n", Servo3New, Servo3);
    }

    if(Servo4New>Servo4){
      moveServo4(1);
      endPosOK=0;
      //printf("ServoNew4=%d, Servo4=%d\n", Servo4New, Servo4);
    }
    if(Servo4New<Servo4){
      moveServo4(-1);
      endPosOK=0;
      //printf("ServoNew4=%d, Servo4=%d\n", Servo4New, Servo4);
    }

    if(Servo5New>Servo5){
      moveServo5(1);
      endPosOK=0;
      //printf("ServoNew5=%d, Servo5=%d\n", Servo5New, Servo5);
    }
    if(Servo5New<Servo5){
      moveServo5(-1);
      endPosOK=0;
      //printf("ServoNew5=%d, Servo5=%d\n", Servo5New, Servo5);
    }
    usleep(StepPause);
  }



  currentX = X;
  currentY = Y;
  currentZ = Z;



  return 0;
}
//-------------------------------------- MoveToCoordinates -----------------------------------------------


//######################################################################
//################## Main ##############################################
//######################################################################

int main(int argc, char **argv){

  long i;
  long stepNumber = 0;
  FILE *commandsFile;
  FILE *ReplayFile;
  char commandsLine[1000];
  char TextLine[1000];
  char *pEnd;
  int stopPlot = 0;
  int coordReadNo = 0;
  long goToX, goToY, goToZ, goToS3, goToS4, goToS5;
  long sleepTime;
  int gripperOld = 90;



  gpioM1dir = mraa_gpio_init(M1_DIR);
  if (gpioM1dir == NULL) {
    fprintf(stderr, "Are you sure that M1_DIR you requested is valid on your platform?");
    exit(1);
  }
  gpioRet = mraa_gpio_dir(gpioM1dir, MRAA_GPIO_OUT);
  if (gpioRet != MRAA_SUCCESS) {
    mraa_result_print(gpioRet);
  }
  mraa_gpio_use_mmaped(gpioM1dir, 1);

  gpioM1step = mraa_gpio_init(M1_STEP);
  if (gpioM1step == NULL) {
    fprintf(stderr, "Are you sure that M1_STEP you requested is valid on your platform?");
    exit(1);
  }
  gpioRet = mraa_gpio_dir(gpioM1step, MRAA_GPIO_OUT);
  if (gpioRet != MRAA_SUCCESS) {
    mraa_result_print(gpioRet);
  }
  mraa_gpio_use_mmaped(gpioM1step, 1);


  gpioM2dir = mraa_gpio_init(M2_DIR);
  if (gpioM2dir == NULL) {
    fprintf(stderr, "Are you sure that M2_DIR you requested is valid on your platform?");
    exit(1);
  }
  gpioRet = mraa_gpio_dir(gpioM2dir, MRAA_GPIO_OUT);
  if (gpioRet != MRAA_SUCCESS) {
    mraa_result_print(gpioRet);
  }
  mraa_gpio_use_mmaped(gpioM2dir, 1);

  gpioM2step = mraa_gpio_init(M2_STEP);
  if (gpioM2step == NULL) {
    fprintf(stderr, "Are you sure that M2_STEP you requested is valid on your platform?");
    exit(1);
  }
  gpioRet = mraa_gpio_dir(gpioM2step, MRAA_GPIO_OUT);
  if (gpioRet != MRAA_SUCCESS) {
    mraa_result_print(gpioRet);
  }
  mraa_gpio_use_mmaped(gpioM2step, 1);


  gpioM3dir = mraa_gpio_init(M3_DIR);
  if (gpioM3dir == NULL) {
    fprintf(stderr, "Are you sure that M3_DIR you requested is valid on your platform?");
    exit(1);
  }
  gpioRet = mraa_gpio_dir(gpioM3dir, MRAA_GPIO_OUT);
  if (gpioRet != MRAA_SUCCESS) {
    mraa_result_print(gpioRet);
  }
  mraa_gpio_use_mmaped(gpioM3dir, 1);

  gpioM3step = mraa_gpio_init(M3_STEP);
  if (gpioM3step == NULL) {
    fprintf(stderr, "Are you sure that M3_STEP you requested is valid on your platform?");
    exit(1);
  }
  gpioRet = mraa_gpio_dir(gpioM3step, MRAA_GPIO_OUT);
  if (gpioRet != MRAA_SUCCESS) {
    mraa_result_print(gpioRet);
  }
  mraa_gpio_use_mmaped(gpioM3step, 1);




  gpioP1a = mraa_gpio_init(MOTOR_P1_A);
  if (gpioP1a == NULL) {
    fprintf(stderr, "Are you sure that MOTOR_P1_A you requested is valid on your platform?");
    exit(1);
  }
  gpioRet = mraa_gpio_dir(gpioP1a, MRAA_GPIO_OUT);
  if (gpioRet != MRAA_SUCCESS) {
    mraa_result_print(gpioRet);
  }
  mraa_gpio_use_mmaped(gpioP1a, 1);

  gpioP1b = mraa_gpio_init(MOTOR_P1_B);
  if (gpioP1b == NULL) {
    fprintf(stderr, "Are you sure that MOTOR_P1_B you requested is valid on your platform?");
    exit(1);
  }
  gpioRet = mraa_gpio_dir(gpioP1b, MRAA_GPIO_OUT);
  if (gpioRet != MRAA_SUCCESS) {
    mraa_result_print(gpioRet);
  }
  mraa_gpio_use_mmaped(gpioP1b, 1);

  gpioP1c = mraa_gpio_init(MOTOR_P1_C);
  if (gpioP1c == NULL) {
    fprintf(stderr, "Are you sure that MOTOR_P1_C you requested is valid on your platform?");
    exit(1);
  }
  gpioRet = mraa_gpio_dir(gpioP1c, MRAA_GPIO_OUT);
  if (gpioRet != MRAA_SUCCESS) {
    mraa_result_print(gpioRet);
  }
  mraa_gpio_use_mmaped(gpioP1c, 1);

  gpioP1d = mraa_gpio_init(MOTOR_P1_D);
  if (gpioP1d == NULL) {
    fprintf(stderr, "Are you sure that MOTOR_P1_D you requested is valid on your platform?");
    exit(1);
  }
  gpioRet = mraa_gpio_dir(gpioP1d, MRAA_GPIO_OUT);
  if (gpioRet != MRAA_SUCCESS) {
    mraa_result_print(gpioRet);
  }
  mraa_gpio_use_mmaped(gpioP1d, 1);



  gpioP2a = mraa_gpio_init(MOTOR_P2_A);
  if (gpioP2a == NULL) {
    fprintf(stderr, "Are you sure that MOTOR_P2_A you requested is valid on your platform?");
    exit(1);
  }
  gpioRet = mraa_gpio_dir(gpioP2a, MRAA_GPIO_OUT);
  if (gpioRet != MRAA_SUCCESS) {
    mraa_result_print(gpioRet);
  }
  mraa_gpio_use_mmaped(gpioP2a, 1);

  gpioP2b = mraa_gpio_init(MOTOR_P2_B);
  if (gpioP2b == NULL) {
    fprintf(stderr, "Are you sure that MOTOR_P2_B you requested is valid on your platform?");
    exit(1);
  }
  gpioRet = mraa_gpio_dir(gpioP2b, MRAA_GPIO_OUT);
  if (gpioRet != MRAA_SUCCESS) {
    mraa_result_print(gpioRet);
  }
  mraa_gpio_use_mmaped(gpioP2b, 1);

  gpioP2c = mraa_gpio_init(MOTOR_P2_C);
  if (gpioP2c == NULL) {
    fprintf(stderr, "Are you sure that MOTOR_P2_C you requested is valid on your platform?");
    exit(1);
  }
  gpioRet = mraa_gpio_dir(gpioP2c, MRAA_GPIO_OUT);
  if (gpioRet != MRAA_SUCCESS) {
    mraa_result_print(gpioRet);
  }
  mraa_gpio_use_mmaped(gpioP2c, 1);

  gpioP2d = mraa_gpio_init(MOTOR_P2_D);
  if (gpioP2d == NULL) {
    fprintf(stderr, "Are you sure that MOTOR_P2_D you requested is valid on your platform?");
    exit(1);
  }
  gpioRet = mraa_gpio_dir(gpioP2d, MRAA_GPIO_OUT);
  if (gpioRet != MRAA_SUCCESS) {
    mraa_result_print(gpioRet);
  }
  mraa_gpio_use_mmaped(gpioP2d, 1);



  gpioEnable = mraa_gpio_init(ENABLE_MOTORS);
  if (gpioEnable == NULL) {
    fprintf(stderr, "Are you sure that ENABLE_MOTORS you requested is valid on your platform?");
    exit(1);
  }
  gpioRet = mraa_gpio_dir(gpioEnable, MRAA_GPIO_OUT);
  if (gpioRet != MRAA_SUCCESS) {
    mraa_result_print(gpioRet);
  }
  gpioRet = mraa_gpio_write(gpioEnable, 1);


  pwmServo3 = mraa_pwm_init(SERVO_3);
  pwmServo4 = mraa_pwm_init(SERVO_4);
  pwmServo5 = mraa_pwm_init(SERVO_5);

  mraa_pwm_enable(pwmServo3,0 );
  mraa_pwm_period_ms(pwmServo3,20);
  mraa_pwm_enable(pwmServo3,1 );

  mraa_pwm_enable(pwmServo4,0 );
  mraa_pwm_period_ms(pwmServo4,20);
  mraa_pwm_enable(pwmServo4,1 );

  mraa_pwm_enable(pwmServo5,0 );
  mraa_pwm_period_ms(pwmServo5,20);
  mraa_pwm_enable(pwmServo5,1 );

  //drive servos to initial position
  mraa_pwm_pulsewidth_us (pwmServo3, (Servo3 + SERVO_3_OFFSET) * 1000 / 90 + 500);
  mraa_pwm_pulsewidth_us (pwmServo4, (Servo4 + SERVO_4_OFFSET) * 1000 / 90 + 500);
  mraa_pwm_pulsewidth_us (pwmServo5, (Servo5 + SERVO_5_OFFSET) * 1000 / 90 + 500);

/*
  pinMode (M1_DIR, OUTPUT);
  pinMode (M1_STEP, OUTPUT);
  pinMode (M2_DIR, OUTPUT);
  pinMode (M2_STEP, OUTPUT);
  pinMode (M3_DIR, OUTPUT);
  pinMode (M3_STEP, OUTPUT);

  pinMode (ENABLE_MOTORS, OUTPUT);

  digitalWrite(ENABLE_MOTORS, 1);

  pinMode (MOTOR_P1_A, OUTPUT);
  pinMode (MOTOR_P1_B, OUTPUT);
  pinMode (MOTOR_P1_C, OUTPUT);
  pinMode (MOTOR_P1_D, OUTPUT);

  pinMode (MOTOR_P2_A, OUTPUT);
  pinMode (MOTOR_P2_B, OUTPUT);
  pinMode (MOTOR_P2_C, OUTPUT);
  pinMode (MOTOR_P2_D, OUTPUT);


  gpioServo(SERVO_3, (Servo3 + SERVO_3_OFFSET) * 1000 / 90 + 500);
  gpioServo(SERVO_4, (Servo4 + SERVO_4_OFFSET) * 1000 / 90 + 500);
  gpioServo(SERVO_5, (Servo5 + SERVO_5_OFFSET) * 1000 / 90 + 500);
*/


  //Calculate XYZ coordinates of motors
  M1X = -BASELENGTH / 2.0;
  M1Y = BASELENGTH / 2.0 * tan(30.0 * PI / 180.0);

  M2X = 0.0;
  M2Y = -(BASELENGTH * cos(30.0 * PI / 180.0) - M1Y);

  M3X = BASELENGTH / 2.0;
  M3Y = BASELENGTH / 2.0 * tan(30.0 * PI / 180.0);

  M2Z = sqrt(pow(INITIAL_LENGTH, 2.0) - pow(M2Y + RADIUS_CORD_MOUNT, 2.0));
  M1Z = M2Z;
  M3Z = M2Z;

  M1X = M1X * StepsPermm;
  M1Y = M1Y * StepsPermm;
  M1Z = M1Z * StepsPermm;

  M2X = M2X * StepsPermm;
  M2Y = M2Y * StepsPermm;
  M2Z = M2Z * StepsPermm;

  M3X = M3X * StepsPermm;
  M3Y = M3Y * StepsPermm;
  M3Z = M3Z * StepsPermm;

  printf("\n\nKoordinates in steps:\n");
  printf("M1X = %lf, M1Y = %lf, M1Z = %lf\n", M1X, M1Y, M1Z);
  printf("M2X = %lf, M2Y = %lf, M2Z = %lf\n", M2X, M2Y, M2Z);
  printf("M3X = %lf, M3Y = %lf, M3Z = %lf\n", M3X, M3Y, M3Z);

  length1 = labs(INITIAL_LENGTH * StepsPermm);
  length2 = labs(INITIAL_LENGTH * StepsPermm);
  length3 = labs(INITIAL_LENGTH * StepsPermm);

  // Simple calculation
  length1 = labs(sqrt(pow(M1X, 2.0) + pow(M1Y, 2.0) + pow(M1Z, 2.0)));
  length2 = labs(sqrt(pow(M2X, 2.0) + pow(M2Y, 2.0) + pow(M2Z, 2.0)));
  length3 = labs(sqrt(pow(M3X, 2.0) + pow(M3Y, 2.0) + pow(M3Z, 2.0)));



  printf("length1 = %lf, length2 = %lf, length3 = %lf\n", length1, length2, length3);
/*
  motor1(1000);
  motor1(-1000);
  sleep(3);
  motor2(1000);
  motor2(-1000);
  sleep(3);
  motor3(1000);
  motor3(-1000);
  sleep(3);
*/

//  MoveToCoordinates(10000, 0, 0, Servo3, Servo4,Servo5);
//  sleep(3);
//  MoveToCoordinates(0, 0, 0, Servo3, Servo4,Servo5);
//  sleep(3);
//  digitalWrite(ENABLE_MOTORS, 0); gpioTerminate(); exit(0);//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

printf("Moving P1 20\n");
for(i=0;i<20;i++){
  motorP1(1, P_PAUSE);
  usleep(20000);
}
printf("...done!\n");
sleep(2);
printf("Moving P1 -20\n");
for(i=0;i<20;i++){
  motorP1(-1, P_PAUSE);
  usleep(20000);
}
printf("...done!\n");

printf("Moving P2 20\n");
for(i=0;i<20;i++){
  motorP2(1, P_PAUSE);
  usleep(20000);
}
printf("...done!\n");
sleep(2);
printf("Moving P2 -20\n");
for(i=0;i<20;i++){
  motorP2(-1, P_PAUSE);
  usleep(20000);
}
printf("...done!\n");

  struct timeval tv;
  unsigned long uTimeNow, longTemp;

    gettimeofday(&tv,NULL);
    uTimeNow = 1000000 * tv.tv_sec + tv.tv_usec;
    longTemp = uTimeNow / 100000000;
    uTimeNow -= longTemp * 100000000;

    printf("uTimeNow=%ld\n", uTimeNow);

    usleep(200000);

    gettimeofday(&tv,NULL);
    uTimeNow = 1000000 * tv.tv_sec + tv.tv_usec;
    longTemp = uTimeNow / 100000000;
    uTimeNow -= longTemp * 100000000;

    printf("uTimeNow=%ld\n", uTimeNow);


  system("rm WinchBotCommands.txt");

  while(run){


    if(access("WinchBotCommands.txt", W_OK) != -1) {
      //sleep(1);
      if (!(commandsFile = fopen("WinchBotCommands.txt", "r"))){
        printf("Can't open WinchBotCommands.txt!\n");
        run = 0;
      }
      else{
        char a = 0;
        i = 0;
        commandsLine[0] = '\0';
        while((!(feof(commandsFile))) && a != 13 && i < 1000){
          fread(&a, 1, 1, commandsFile);
          commandsLine[i] = a;
          commandsLine[i+1] = '\0';
          i++;
          //printf("'%c'", a);
        }
        printf("commandsLine='%s'\n", commandsLine);

        //Store current coordinates in file
        if(strstr(commandsLine, "STORE")!=NULL){
          sprintf(TextLine, "echo \"% 7ld, % 7ld, % 7ld, % 7d, % 7d, % 7d,        0;\" >> coordinates.bot", currentX, currentY, currentZ, Servo3, Servo4, Servo5);
          system(TextLine);
        }

        //Store current coordinates in file
        if(strstr(commandsLine, "DELETEALL")!=NULL){
          char tmpCommand[1000];
          sprintf(tmpCommand, "cp coordinates.bot backup/coordinates-%ld.bot", time(0));
          system(tmpCommand);
          system("rm coordinates.bot");
        }

        //Go to initial position
        if(strstr(commandsLine, "INIT")!=NULL){
          MoveToCoordinates(0, 0, 0, 90, 90, 90, 0);
        }

        //Replay file
        if(strstr(commandsLine, "REPLAY")!=NULL){
          if((ReplayFile=fopen("coordinates.bot","rb"))!=NULL){
            printf("Replaying coordinates.bot\n");
            //PlotStartTime = time(0);
            //coordinateCount = 0;
            stopPlot = 0;
            coordReadNo = 0;

            while(!(feof(ReplayFile)) && stopPlot == 0){
              fread(&a, 1, 1, ReplayFile);
              printf("%c", a);
              i=0;
              TextLine[0] = '\0';
              while(!(feof(ReplayFile)) && a !=' ' && a != ';' && a != '\"' && a != ',' && a != 10 && a!= 13){
                TextLine[i] = a;
                TextLine[i+1] = '\0';
                i++;
                fread(&a, 1, 1, ReplayFile);
                printf("%c", a);
              }
              if(a == '#'){
                while(!(feof(ReplayFile)) && a != 10 && a!= 13){
                  fread(&a, 1, 1, ReplayFile);
                  printf("%c", a);
                }
              }
              if(a == ',' || a == ';'){//Coordinates found
                coordReadNo++;
                if(coordReadNo == 1){
                  goToX = strtol(TextLine, &pEnd, 10);
                }
                if(coordReadNo == 2){
                  goToY = strtol(TextLine, &pEnd, 10);
                }
                if(coordReadNo == 3){
                  goToZ = strtol(TextLine, &pEnd, 10);
                }
                if(coordReadNo == 4){
                  goToS3 = strtol(TextLine, &pEnd, 10);
                }
                if(coordReadNo == 5){
                  goToS4 = strtol(TextLine, &pEnd, 10);
                }
                if(coordReadNo == 6){
                  goToS5 = strtol(TextLine, &pEnd, 10);
                }
                if(coordReadNo == 7){
                  sleepTime = strtol(TextLine, &pEnd, 10);
                  if(sleepTime < 0){
                    StepPause = 3000L;
                    sleepTime = 0;
                  }
                  else{
                    StepPause = STEP_PAUSE;
                  }
                  printf("X=%ld, Y=%ld, Z=%ld, S3=%ld, S4=%ld, S5=%ld, sleep=%ld", goToX, goToY, goToZ, goToS3, goToS4, goToS5, sleepTime);
                  coordReadNo = 0;
                  if(gripperOld < goToS5){//Open gripper slowly
                    while(Servo5 < goToS5){
                      moveServo5(1);
                      //usleep(200000);
                    }
                    sleep(1);//wait after opening gripper
                  }
                  if(gripperOld > goToS5){//Close gripper slowly
                    sleep(1);//wait before closing gripper
                    while(Servo5 > goToS5){
                      moveServo5(-1);
                      //usleep(200000);
                    }
                  }
                  MoveToCoordinates(goToX, goToY, goToZ, goToS3, goToS4, goToS5, 0);
                  gripperOld = goToS5;
                  sleep(sleepTime);
                  //sleep(3);// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                  //coordinateCount++;
                }

              }
            }//while(!(feof(ReplayFile)) && stopPlot == 0){
            fclose(ReplayFile);
            StepPause = STEP_PAUSE;
            printf("...done!\n");
          }
        }//if(strstr(commandsLine, "REPLAY")!=NULL)



        //Platform control
        if(strstr(commandsLine, "P1,")!=NULL){
          stepNumber = strtol(commandsLine + 3, &pEnd, 10);
          printf("motorP1(%ld)\n", stepNumber);
          for(i=0;i<labs(stepNumber);i++){
            if(stepNumber > 0){
              motorP1(1, P_PAUSE);
            }
            else{
              motorP1(-1, P_PAUSE);
            }
          }
          printf("P1 moved!\n");
          AngleP1 = 0.0;
          AngleP2 = 0.0;
        }

        //Platform control
        if(strstr(commandsLine, "P2,")!=NULL){
          stepNumber = strtol(commandsLine + 3, &pEnd, 10);
          printf("motorP2(%ld)\n", stepNumber);
          for(i=0;i<labs(stepNumber);i++){
            if(stepNumber > 0){
              motorP2(1, P_PAUSE);
            }
            else{
              motorP2(-1, P_PAUSE);
            }
            usleep(20000);
          }
          printf("P2 moved!\n");
          AngleP1 = 0.0;
          AngleP2 = 0.0;
        }


        //Motor control
        if(strstr(commandsLine, "M1,")!=NULL){
          stepNumber = strtol(commandsLine + 3, &pEnd, 10);
          for(i=0;i<labs(stepNumber);i++){
            if(stepNumber > 0){
              motor1(1, STEP_PAUSE);
            }
            else{
              motor1(-1, STEP_PAUSE);
            }
          }
          printf("M1 moved %ld!\n", stepNumber);
          currentX = 0;
          currentY = 0;
          currentZ = 0;
          length1 = labs(INITIAL_LENGTH * StepsPermm);
          length2 = labs(INITIAL_LENGTH * StepsPermm);
          length3 = labs(INITIAL_LENGTH * StepsPermm);
          // Simple calculation
          length1 = labs(sqrt(pow(M1X, 2.0) + pow(M1Y, 2.0) + pow(M1Z, 2.0)));
          length2 = labs(sqrt(pow(M2X, 2.0) + pow(M2Y, 2.0) + pow(M2Z, 2.0)));
          length3 = labs(sqrt(pow(M3X, 2.0) + pow(M3Y, 2.0) + pow(M3Z, 2.0)));

        }
        if(strstr(commandsLine, "M2,")!=NULL){
          stepNumber = strtol(commandsLine + 3, &pEnd, 10);
          for(i=0;i<labs(stepNumber);i++){
            if(stepNumber > 0){
              motor2(1, STEP_PAUSE);
            }
            else{
              motor2(-1, STEP_PAUSE);
            }
          }
          printf("M2 moved!\n");
          currentX = 0;
          currentY = 0;
          currentZ = 0;
          length1 = labs(INITIAL_LENGTH * StepsPermm);
          length2 = labs(INITIAL_LENGTH * StepsPermm);
          length3 = labs(INITIAL_LENGTH * StepsPermm);
          // Simple calculation
          length1 = labs(sqrt(pow(M1X, 2.0) + pow(M1Y, 2.0) + pow(M1Z, 2.0)));
          length2 = labs(sqrt(pow(M2X, 2.0) + pow(M2Y, 2.0) + pow(M2Z, 2.0)));
          length3 = labs(sqrt(pow(M3X, 2.0) + pow(M3Y, 2.0) + pow(M3Z, 2.0)));
        }
        if(strstr(commandsLine, "M3,")!=NULL){
          stepNumber = strtol(commandsLine + 3, &pEnd, 10);
          for(i=0;i<labs(stepNumber);i++){
            if(stepNumber > 0){
              motor3(1, STEP_PAUSE);
            }
            else{
              motor3(-1, STEP_PAUSE);
            }
          }
          printf("M3 moved!\n");
          currentX = 0;
          currentY = 0;
          currentZ = 0;
          length1 = labs(INITIAL_LENGTH * StepsPermm);
          length2 = labs(INITIAL_LENGTH * StepsPermm);
          length3 = labs(INITIAL_LENGTH * StepsPermm);
          // Simple calculation
          length1 = labs(sqrt(pow(M1X, 2.0) + pow(M1Y, 2.0) + pow(M1Z, 2.0)));
          length2 = labs(sqrt(pow(M2X, 2.0) + pow(M2Y, 2.0) + pow(M2Z, 2.0)));
          length3 = labs(sqrt(pow(M3X, 2.0) + pow(M3Y, 2.0) + pow(M3Z, 2.0)));
        }

        //Servo control
        if(strstr(commandsLine, "Servo3,")!=NULL){
          stepNumber = strtol(commandsLine + 7, &pEnd, 10);
          MoveToCoordinates(currentX, currentY, currentZ, Servo3 + stepNumber, Servo4, Servo5, 0);
          //moveServo3(stepNumber);
        }

        if(strstr(commandsLine, "Servo4,")!=NULL){
          stepNumber = strtol(commandsLine + 7, &pEnd, 10);
          MoveToCoordinates(currentX, currentY, currentZ, Servo3, Servo4 + stepNumber, Servo5, 0);
          //moveServo4(stepNumber);
        }

        if(strstr(commandsLine, "Servo5,")!=NULL){
          stepNumber = strtol(commandsLine + 7, &pEnd, 10);
          MoveToCoordinates(currentX, currentY, currentZ, Servo3, Servo4, Servo5 + stepNumber, 0);
          //moveServo5(stepNumber);
        }

        //Axis control
        if(strstr(commandsLine, "X,")!=NULL){
          stepNumber = strtol(commandsLine + 2, &pEnd, 10);
          MoveToCoordinates(currentX + stepNumber, currentY, currentZ, Servo3, Servo4, Servo5, 0);
        }
        if(strstr(commandsLine, "Y,")!=NULL){
          stepNumber = strtol(commandsLine + 2, &pEnd, 10);
          MoveToCoordinates(currentX, currentY + stepNumber, currentZ, Servo3, Servo4, Servo5, 0);
        }
        if(strstr(commandsLine, "Z,")!=NULL){
          stepNumber = strtol(commandsLine + 2, &pEnd, 10);
          MoveToCoordinates(currentX, currentY, currentZ + stepNumber, Servo3, Servo4, Servo5, 0);
        }
      }
      system("rm WinchBotCommands.txt");
    }
  }



  gpioRet = mraa_gpio_write(gpioEnable, 0);

  gpioRet = mraa_gpio_write(gpioP1a, 0);
  gpioRet = mraa_gpio_write(gpioP1b, 0);
  gpioRet = mraa_gpio_write(gpioP1c, 0);
  gpioRet = mraa_gpio_write(gpioP1d, 0);

  gpioRet = mraa_gpio_write(gpioP2a, 0);
  gpioRet = mraa_gpio_write(gpioP2b, 0);
  gpioRet = mraa_gpio_write(gpioP2c, 0);
  gpioRet = mraa_gpio_write(gpioP2d, 0);

  return 0;
}



